@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000-2016 Erwin Waterlander
+Copyright (C) 2000-2019 Erwin Waterlander
 
 Ideas and source code of NCD (Ninux Czo Directory) have been
 used in the WCD graphical interface.
@@ -35,7 +35,6 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 #include <stdio.h>
 #include <string.h>
-#include <signal.h>
 #if defined(WCD_UNICODE) || (defined(_WIN32) && !defined(__CYGWIN__))
 #ifndef __USE_XOPEN
 #define __USE_XOPEN
@@ -53,7 +52,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
 #include "wcd.h"
-#include "dosdir.h"
+#include "finddirs.h"
 #include "display.h"
 #include "wfixpath.h"
 #include "match.h"
@@ -65,6 +64,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include "config.h"
 #ifdef WCD_USECURSES
 #include "colors.h"             /* add colors for the tree on MS platform */
+#ifndef KEY_RESIZE
+#include <signal.h>
+#endif
 #endif
 
 /*
@@ -108,7 +110,7 @@ static const char WCD_COMPACT_MOREDIR[] =   { 32, WCD_ACS_VL, WCD_SPACE,  WCD_SP
 static const char WCD_COMPACT_ENDDIR[] =    { 32, WCD_ACS_LLC, WCD_ACS_HL, WCD_ACS_HL, 0} ;
 #endif
 
-#define WCD_GRAPH_MAX_LINE_LENGTH  DD_MAXPATH * 2
+#define WCD_GRAPH_MAX_LINE_LENGTH  WCD_MAXPATH * 2
 
 #ifdef WCD_USECURSES
 
@@ -142,9 +144,18 @@ void dataRefresh(int ydiff, int init);
  * to just rebuild the windows from scratch */
 void ioResize()
 {
-   /* end curses mode */
-   endwin();
-   refresh();  /* start curses */
+#ifdef __PDCURSES__
+   /* Correct resizing on Windows (ConEmu and Windows 10 console) requires a
+      recent PDCurses version ( >= 3.7 ). */
+   resize_term(0,0);
+#else  
+   /* resize_term() is not working properly with old versions of curses.
+      (e.g. seen with ncurses 5.6.20080804). There are still a lot of old
+      curses library installations around. Stopping and restarting curses
+      does the job too with old curses libraries. */
+   endwin();  /* end curses mode */
+   refresh(); /* start curses */
+#endif
 
    wcd_cwin.scrollWinHeight = LINES - INPUT_WIN_HEIGHT;
 
@@ -167,6 +178,7 @@ void ioResize()
    dataRefresh(0, 1);
 }
 
+#ifndef KEY_RESIZE
 #if CAN_RESIZE
 void signalSigwinch (int sig)
 {
@@ -174,6 +186,7 @@ void signalSigwinch (int sig)
   signal(SIGWINCH,signalSigwinch);
 }
 #endif
+#endif
 
 #endif /* WCD_USECURSES */
 
@@ -586,7 +599,7 @@ void addPath(text path, dirnode d)
    dirnode n,n_up;
    text t;
 #if (defined(_WIN32) || defined(__CYGWIN__))
-   static char buf[DD_MAXPATH] = "//" ;
+   static char buf[WCD_MAXPATH] = "//" ;
 
    if ( (path != NULL) &&
         (*path == '/') &&
@@ -677,8 +690,8 @@ void ssort_dirnode (dirnode* subdirs, int left, int right)
 {
  int i, last;
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
- static wchar_t wstr_left[DD_MAXPATH];
- static wchar_t wstr_right[DD_MAXPATH];
+ static wchar_t wstr_left[WCD_MAXPATH];
+ static wchar_t wstr_right[WCD_MAXPATH];
 #endif
  text t_left, t_right;
 
@@ -692,8 +705,8 @@ void ssort_dirnode (dirnode* subdirs, int left, int right)
    t_left  = (subdirs[left])->name;
    t_right = (subdirs[i])->name;
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
-   size_t len1 = MBSTOWCS(wstr_left, t_left, (size_t)DD_MAXPATH);
-   size_t len2 = MBSTOWCS(wstr_right, t_right, (size_t)DD_MAXPATH);
+   size_t len1 = MBSTOWCS(wstr_left, t_left, (size_t)WCD_MAXPATH);
+   size_t len2 = MBSTOWCS(wstr_right, t_right, (size_t)WCD_MAXPATH);
    if ((len1 == (size_t)(-1)) || (len2 == (size_t)(-1)))
    {
       /* Erroneous multi-byte sequence */
@@ -818,7 +831,7 @@ void buildTreeFromFile(char* filename, dirnode d, int quiet)
        while (!feof(infile) && !ferror(infile))  /* parse the file */
        {
           int len;
-          char line[DD_MAXPATH];
+          char line[WCD_MAXPATH];
 
           len = read_treefile_line(line,infile,filename,&line_nr, bomtype);
           if (ferror(infile)) {
@@ -851,7 +864,7 @@ dirnode createRootNode()
 
    if (rootNode != NULL)
    {
-      dirnodeSetName(ROOTDIR,rootNode);
+      dirnodeSetName(textNew(ROOTDIR),rootNode);
       dirnodeSetX(0,rootNode);
       dirnodeSetY(0,rootNode);
    }
@@ -873,7 +886,7 @@ dirnode searchNodeForDir(text path, dirnode d, dirnode rNode)
    dirnode n;
    dirnode rootnode;
 #if (defined(_WIN32) || defined(__CYGWIN__))
-   static char buf[DD_MAXPATH] = "//" ;
+   static char buf[WCD_MAXPATH] = "//" ;
 #endif
 
    if (d == NULL)
@@ -936,7 +949,7 @@ char *getNodeFullPath(dirnode node)
 
    /* The reconstructed path will not be longer than
     * it was original in the treedata file. The max length
-    * is DD_MAXPATH. See also function `read_treefile()'
+    * is WCD_MAXPATH. See also function `read_treefile()'
     *
     * The reconstructed path has only an extra slash "/"
     * in the beginning on DOS/Windows if environment
@@ -945,11 +958,11 @@ char *getNodeFullPath(dirnode node)
 
    if (line == NULL)
    {
-      line = textNewSize((size_t)(DD_MAXPATH+1));
+      line = textNewSize((size_t)(WCD_MAXPATH+1));
    }
    if (tline == NULL)
    {
-      tline = textNewSize((size_t)(DD_MAXPATH+1));
+      tline = textNewSize((size_t)(WCD_MAXPATH+1));
    }
 
    line[0] = '\0';
@@ -1555,8 +1568,8 @@ void updateLine(WINDOW *win, dirnode n, int i, int y, dirnode curNode, int xoffs
    {
       int j;
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
-      static wchar_t wstr[DD_MAXPATH];
-      size_t len = MBSTOWCS(wstr,(char *)s,(size_t)DD_MAXPATH); /* number of wide characters */
+      static wchar_t wstr[WCD_MAXPATH];
+      size_t len = MBSTOWCS(wstr,(char *)s,(size_t)WCD_MAXPATH); /* number of wide characters */
 #else
       size_t len = strlen((char *)s);
 #endif
@@ -1788,10 +1801,9 @@ char *getZoomStackPath(dirnode stack)
    size_t i, size;
    text name;
 
-
    if (line == NULL)
    {
-      line = textNewSize((size_t)DD_MAXPATH);
+      line = textNewSize((size_t)WCD_MAXPATH);
    }
 
    line[0] = '\0';
@@ -1803,11 +1815,10 @@ char *getZoomStackPath(dirnode stack)
       if(i != 0)
          strcat(line,"/");
       name = dirnodeGetName(elementAtDirnode(i,stack));
-      if((strlen(line)+strlen(name)) < (size_t)DD_MAXPATH)
+      if((strlen(line)+strlen(name)) < (size_t)WCD_MAXPATH)
          strcat(line,name);
    }
 
-
    return(line);
 }
 
@@ -1875,19 +1886,19 @@ void dataRefresh(int ydiff, int init)
   }
 
   /* mvwprintw(inputWin, 1,0,"%s",getNodeFullPath(curNode)); */
-  s = (wcd_uchar *)getZoomStackPath(wcd_cwin.zoomStack); /* s has size DD_MAXPATH */
+  s = (wcd_uchar *)getZoomStackPath(wcd_cwin.zoomStack); /* s has size WCD_MAXPATH */
   strcat((char *)s, getNodeFullPath(wcd_cwin.curNode));
-  wcd_fixpath((char *)s, (size_t)DD_MAXPATH);
+  wcd_fixpath((char *)s, (size_t)WCD_MAXPATH);
 
   if (s != NULL)
   {
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
-   static wchar_t wstr[DD_MAXPATH];  /* Declarations at beginning of scope for Watcom C */
+   static wchar_t wstr[WCD_MAXPATH];  /* Declarations at beginning of scope for Watcom C */
    int width;
 #endif
     wmove(wcd_cwin.inputWin, 1, 0);
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
-   len = (int)MBSTOWCS(wstr,(char *)s,(size_t)DD_MAXPATH); /* number of wide characters */
+   len = (int)MBSTOWCS(wstr,(char *)s,(size_t)WCD_MAXPATH); /* number of wide characters */
    if (len < 0)
    {
       /* Erroneous multi-byte sequence */
@@ -2090,7 +2101,7 @@ dirnode popZoom(dirnode zoomStack, dirnode curNode, int *ymax)
    dirnodeSetDown(dirnodeGetDown(zlast),top);
 
    /* remove last element from stack */
-   removeElementAtDirnode(size - 1, zoomStack, false, false);
+   removeElementAtDirnode(size - 1, zoomStack, true, true);
 
    newtop = endOfRecursionOfDirnodeParent(top);
    dirnodeSetX(0,newtop);
@@ -2236,11 +2247,9 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
 {
   int c = 0, n =0, y, ymax;
   int ydiff;
-  char curPath[DD_MAXPATH];
+  char curPath[WCD_MAXPATH];
   char *ptr, *ptr2;
-#ifndef __PDCURSES__
   SCREEN *sp;
-#endif
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
   wint_t ch;
 #endif
@@ -2254,7 +2263,7 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
 
    ptr=NULL;
 
-   if (getCurPath(curPath, (size_t)DD_MAXPATH, use_HOME) == NULL)
+   if (getCurPath(curPath, (size_t)WCD_MAXPATH, use_HOME) == NULL)
       return NULL;
    wcd_cwin.curNode = locatePathOrSo(curPath,tree);
    if (wcd_cwin.curNode == NULL)
@@ -2265,22 +2274,21 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
 
    wcd_cwin.zoomStack = dirnodeNew(NULL,NULL,NULL);
 
+#ifndef KEY_RESIZE
 #if CAN_RESIZE
    signal (SIGWINCH, signalSigwinch);
 #endif
+#endif
 
 /* Older versions of PDCurses and ncurses < 5.9.20120922 do not
  * support newterm() on Windows */
-#if defined(__PDCURSES__) || (defined(_WIN32) && !defined(__CYGWIN__))
-   initscr();
-#else
    sp = newterm(NULL,stdout,stdin);
    if (sp == NULL)
    {
+      freeDirnode(wcd_cwin.zoomStack,true);
       print_error("%s", _("Error opening terminal, falling back to stdout interface.\n"));
       return NULL;
    }
-#endif
 
    if (wcd_cwin.graphics_mode & WCD_GRAPH_COLOR)
       initcolor();
@@ -2295,16 +2303,11 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
 
    wcd_cwin.scrollWinHeight = LINES - INPUT_WIN_HEIGHT;
    wcd_cwin.scrollWin = newwin(wcd_cwin.scrollWinHeight,COLS,0,0);
-   wcd_cwin.inputWin = newwin(INPUT_WIN_HEIGHT,COLS,wcd_cwin.scrollWinHeight,0);
-
-   if (wcd_cwin.graphics_mode & WCD_GRAPH_COLOR)
-   {
-      colorbox (wcd_cwin.scrollWin, MENU_COLOR, 0);     /* BillyC add colors */
-      colorbox (wcd_cwin.inputWin,  BODY_COLOR, 0);     /* BillyC add colors */
-   }
    if (wcd_cwin.scrollWin == NULL)
    {
       endwin();
+      delscreen(sp);
+      freeDirnode(wcd_cwin.zoomStack,true);
 #ifdef XCURSES
       XCursesExit();
 #endif
@@ -2314,6 +2317,26 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
 
    scrollok(wcd_cwin.scrollWin, TRUE);
 
+   wcd_cwin.inputWin = newwin(INPUT_WIN_HEIGHT,COLS,wcd_cwin.scrollWinHeight,0);
+   if (wcd_cwin.inputWin == NULL)
+   {
+      delwin(wcd_cwin.scrollWin);
+      endwin();
+      delscreen(sp);
+      freeDirnode(wcd_cwin.zoomStack,true);
+#ifdef XCURSES
+      XCursesExit();
+#endif
+      print_error("%s", _("error creating input window.\n"));
+      return NULL;
+   }
+
+   if (wcd_cwin.graphics_mode & WCD_GRAPH_COLOR)
+   {
+      colorbox (wcd_cwin.scrollWin, MENU_COLOR, 0);     /* BillyC add colors */
+      colorbox (wcd_cwin.inputWin,  BODY_COLOR, 0);     /* BillyC add colors */
+   }
+
    wcd_cwin.str[n] = '\0';
 
    refresh();
@@ -2651,6 +2674,9 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
          break;
       case KEY_F (5):
       case Key_CTRL ('l'):
+#ifdef KEY_RESIZE
+      case KEY_RESIZE:
+#endif
             ioResize();
          break;
       case Key_CTRL ('i'):
@@ -2668,25 +2694,17 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
             wcd_cwin.mode = WCD_NAV;
          else
             c = 13;
-      break;
+        break;
      case 13: /* Enter */
      case KEY_ENTER:
             c = 13;
-            ptr = getZoomStackPath(wcd_cwin.zoomStack); /* s has size DD_MAXPATH */
+            ptr = getZoomStackPath(wcd_cwin.zoomStack); /* s has size WCD_MAXPATH */
             strcat(ptr,getNodeFullPath(wcd_cwin.curNode));
-            wcd_fixpath(ptr, (size_t)DD_MAXPATH);
-      break;
+            wcd_fixpath(ptr, (size_t)WCD_MAXPATH);
+        break;
      case 8:  /* backspace */
-     case KEY_BACKSPACE:
-            if(n==1)
-               wcd_cwin.mode = WCD_NAV;
-            if(n>1) n--;
-            wcd_cwin.str[n] = '\0';
-#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
-            wcd_cwin.wstr[n] = '\0';
-#endif
-     break;
      case 127: /* delete */
+     case KEY_BACKSPACE:
      case KEY_DC:
             if(n==1)
                wcd_cwin.mode = WCD_NAV;
@@ -2695,7 +2713,7 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
 #if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
             wcd_cwin.wstr[n] = '\0';
 #endif
-     break;
+        break;
      default:
       if ((wcd_cwin.mode == WCD_SEARCH) && (n < (WCD_MAX_INPSTR -1)))
       {
@@ -2728,6 +2746,14 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
 
     if (wcd_cwin.curNode == NULL)
     {
+       delwin(wcd_cwin.scrollWin);
+       delwin(wcd_cwin.inputWin);
+       endwin();
+       delscreen(sp);
+       freeDirnode(wcd_cwin.zoomStack,true);
+#ifdef XCURSES
+       XCursesExit();
+#endif
        print_error(_("Cannot find the current path %s in the directory tree.\n"), curPath);
        return NULL;
     }
@@ -2739,7 +2765,11 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
        dataRefresh(ydiff, 0);
    }
 
+   delwin(wcd_cwin.scrollWin);
+   delwin(wcd_cwin.inputWin);
    endwin();
+   delscreen(sp);
+   freeDirnode(wcd_cwin.zoomStack,true);
 #ifdef XCURSES
    XCursesExit();
 #endif
@@ -2750,11 +2780,10 @@ char *selectANode(dirnode tree, int *use_HOME, int ignore_case, int graphics_mod
    /* remove '/' before drive letter */
    if ((ptr != NULL)&&(*use_HOME)&&
          (*(ptr+1) != '/'))  /* UNC path */
-      ptr++;
+      trimPath(ptr,1);
 #endif
-   wcd_fixpath(ptr, (size_t)DD_MAXPATH);
+   wcd_fixpath(ptr, (size_t)WCD_MAXPATH);
    return(ptr);
-
 }
 
 #endif /* WCD_USECURSES */
