@@ -88,10 +88,11 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include <wctype.h>
 #ifdef WCD_UNINORM
 #  include <uninorm.h>  /* part of libunistring */
+#  include <stdint.h>
 #endif
 #include "wcd.h"
 #include "display.h"
-#include "dosdir.h"
+#include "finddirs.h"
 #include "matchw.h" /* define ToLower() in here (for Unix, define ToLower
                        * to be macro (using isupper()); otherwise just use
                        * tolower() */
@@ -185,12 +186,12 @@ static int recmatchwcs(wchar_t *pattern, wchar_t *string, int ignore_case, wint_
 
 int dd_matchmbs(const char *string, const char *pattern, int ignore_case, int ignore_diacritics)
 {
-    static wchar_t wstr_string[DD_MAXPATH];
-    static wchar_t wstr_pattern[DD_MAXPATH];
+    static wchar_t wstr_string[WCD_MAXPATH];
+    static wchar_t wstr_pattern[WCD_MAXPATH];
     size_t len1,len2;
 
-    len1 = MBSTOWCS(wstr_string , string ,(size_t)DD_MAXPATH);
-    len2 = MBSTOWCS(wstr_pattern, pattern,(size_t)DD_MAXPATH);
+    len1 = MBSTOWCS(wstr_string , string ,(size_t)WCD_MAXPATH);
+    len2 = MBSTOWCS(wstr_pattern, pattern,(size_t)WCD_MAXPATH);
     if ((len1 == (size_t)(-1)) || (len2 == (size_t)(-1)))
         return 0;
     return(dd_matchwcs(wstr_string, wstr_pattern, ignore_case, ignore_diacritics));
@@ -206,25 +207,25 @@ int dd_matchwcs(const wchar_t *string,const wchar_t *pattern,int ignore_case, in
     int result;
 
 #ifdef WCD_UNINORM
-    static wchar_t string_buffer[DD_MAXPATH];
-    static wchar_t pattern_buffer[DD_MAXPATH];
-    size_t lengthp = DD_MAXPATH;
+    static wchar_t string_buffer[WCD_MAXPATH];
+    static wchar_t pattern_buffer[WCD_MAXPATH];
+    size_t lengthp = WCD_MAXPATH;
     wchar_t *string_normalized;
     wchar_t *pattern_normalized;
 
 #  if defined(_WIN32) || defined(__CYGWIN__)
     /* Normalization. Composition, such that we can ignore diacritics. */
-    string_normalized  = u16_normalize (UNINORM_NFKC, string,  wcslen(string) +1, string_buffer,  &lengthp);
+    string_normalized  = (wchar_t*)u16_normalize (UNINORM_NFKC, (uint16_t*)string,  wcslen(string) +1, (uint16_t*)string_buffer,  &lengthp);
     if (string_normalized == NULL)
         return(0);
-    pattern_normalized = u16_normalize (UNINORM_NFKC, pattern, wcslen(pattern) +1, pattern_buffer, &lengthp);
+    pattern_normalized = (wchar_t*)u16_normalize (UNINORM_NFKC, (uint16_t*)pattern, wcslen(pattern) +1, (uint16_t*)pattern_buffer, &lengthp);
     if (pattern_normalized == NULL)
         return(0);
 #  else
-    string_normalized  = u32_normalize (UNINORM_NFKC, string,  wcslen(string) +1, string_buffer,  &lengthp);
+    string_normalized  = (wchar_t*)u32_normalize (UNINORM_NFKC, (uint32_t*)string,  wcslen(string) +1, (uint32_t*)string_buffer,  &lengthp);
     if (string_normalized == NULL)
         return(0);
-    pattern_normalized = u32_normalize (UNINORM_NFKC, pattern, wcslen(pattern) +1, pattern_buffer, &lengthp);
+    pattern_normalized = (wchar_t*)u32_normalize (UNINORM_NFKC, (uint32_t*)pattern, wcslen(pattern) +1, (uint32_t*)pattern_buffer, &lengthp);
     if (pattern_normalized == NULL)
         return(0);
 #  endif
@@ -300,7 +301,7 @@ static int recmatchwcs(wchar_t *p,wchar_t *s,int ic, wint_t *CPTable)
     int result;
 
     /* Get first character, the pattern for new recmatch calls follows */
-    c = *p++;
+    c = (wint_t)*p++;
 
     /* If that was the end of the pattern, match if string empty too */
     if (c == 0)
@@ -352,7 +353,7 @@ static int recmatchwcs(wchar_t *p,wchar_t *s,int ic, wint_t *CPTable)
             if (e == 0 && *p == L'\\')             /* set escape flag if \ */
                 e = 1;
             else if (e == 0 && *p == L'-')         /* set start of range if - */
-                c = *(p-1);
+                c = (wint_t)*(p-1);
             else {
                 wint_t cc = Case((wint_t)*s);
 
@@ -374,7 +375,7 @@ static int recmatchwcs(wchar_t *p,wchar_t *s,int ic, wint_t *CPTable)
 #endif /* !VMS */
 
     /* if escape ('\'), just compare next character */
-    if (c == '\\' && (c = *p++) == 0)     /* if \ at end, then syntax error */
+    if (c == '\\' && (c = (wint_t)*p++) == 0)     /* if \ at end, then syntax error */
         return 0;
 
     /* just a character--compare it */
