@@ -8,7 +8,7 @@ Author: Erwin Waterlander
 ======================================================================
 = Copyright                                                          =
 ======================================================================
-Copyright (C) 2002-2016 Erwin Waterlander
+Copyright (C) 2002-2017 Erwin Waterlander
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -44,15 +44,23 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #if defined(UNIX) || defined(__DJGPP__) || defined(__OS2__)
 # include <unistd.h>
 #endif
-#include "dosdir.h"
+#include "finddirs.h"
 #include "wcddir.h"
 #include "match.h"
 #include <string.h>
 #include <errno.h>
+#include "wcd.h"
+
+#ifdef _WCD_DOSFS
+#  include <direct.h>
+#endif
 
 #if (defined(_WIN32) && defined(WCD_UNICODE))
 #include <wchar.h>
 #endif
+#if (defined(__MSDOS__) && !defined(__DJGPP__)) || (defined(__OS2__) && !defined(__EMX__))
+#  include <io.h> /* for unlink() */
+#endif
 
 #if (defined(_WIN32) || defined(__CYGWIN__))
 
@@ -67,12 +75,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include "Text.h"
 #include "nameset.h"
 #include "wfixpath.h"
-#include "dosdir.h"
 #include <string.h>
 #include "display.h"
 #include <windows.h>
-/* #include <strsafe.h> */
-#include "wcd.h"
 
 
 /*
@@ -118,18 +123,16 @@ int doEnum( int level, NETRESOURCE *pnr, nameset n )
    DWORD count, bufsize, ui;
    NETRESOURCE buf[MAX_NR_BUF];
    /* const char *type; */
-   char path[DD_MAXPATH];
+   char path[WCD_MAXPATH];
 
    rc = WNetOpenEnum( RESOURCE_GLOBALNET, RESOURCETYPE_DISK, 0, pnr, &hEnum );
-   if ( rc == ERROR_ACCESS_DENIED )
-   {
+   if ( rc == ERROR_ACCESS_DENIED ) {
       /* printf( "%-6.6s %-4.4s%*s  Error 5 -- access denied\n", "", "", level * 2, "" ); */
       print_msg( _("access denied.\n"));
       return 1;
    }
 
-   if ( rc )
-   {
+   if ( rc ) {
       rc2 = GetLastError();
       /* printf( "WNOE(): rc = %lu, gle = %lu\n", rc, rc2 ); */
       if ( rc2 == ERROR_EXTENDED_ERROR )
@@ -137,17 +140,14 @@ int doEnum( int level, NETRESOURCE *pnr, nameset n )
       return 0;
    }
 
-   while ( 1 )
-   {
+   while ( 1 ) {
       count = (DWORD) -1L;
       bufsize = sizeof buf;
       rc = WNetEnumResource( hEnum, &count, buf, &bufsize );
       if ( rc != NO_ERROR )
          break;
-      for ( ui = 0; (ui < count); ++ ui )
-      {
-         switch ( buf[ui].dwDisplayType )
-         {
+      for ( ui = 0; (ui < count); ++ ui ) {
+         switch ( buf[ui].dwDisplayType ) {
             case RESOURCEDISPLAYTYPE_DOMAIN:
                /* type = "domain"; */ break;
             case RESOURCEDISPLAYTYPE_GENERIC:
@@ -157,8 +157,8 @@ int doEnum( int level, NETRESOURCE *pnr, nameset n )
             case RESOURCEDISPLAYTYPE_SHARE:
                /* type = "share"; */
                print_msg( "%s\n", buf[ui].lpRemoteName );
-               wcd_strncpy(path, buf[ui].lpRemoteName, DD_MAXPATH);
-               wcd_fixpath(path, DD_MAXPATH);
+               wcd_strncpy(path, buf[ui].lpRemoteName, WCD_MAXPATH);
+               wcd_fixpath(path, WCD_MAXPATH);
                addToNamesetArray(textNew(path), n);
                break;
             default:
@@ -170,8 +170,7 @@ int doEnum( int level, NETRESOURCE *pnr, nameset n )
       }
    }
 
-   if ( rc != ERROR_NO_MORE_ITEMS )  /* bad things */
-   {
+   if ( rc != ERROR_NO_MORE_ITEMS ) { /* bad things */
       rc2 = GetLastError();
       /* printf( "WNER(): rc = %lu, gle = %lu\n", rc, rc2 ); */
       if ( rc2 == ERROR_EXTENDED_ERROR )
@@ -198,8 +197,7 @@ void wcd_getshares(char* path, nameset n)
    if ((path == NULL) || (n == NULL))
       return;
 
-   if (wcd_isServerPath(path))
-   {
+   if (wcd_isServerPath(path)) {
       /* an UNC path, possibly pointing to a server */
       /* WIN32 API needs backslashes. */
       *path = '\\';
@@ -264,18 +262,14 @@ void wcd_PrintError(DWORD dw)
 int wcd_isSharePath (char* path)
 {
    /* Assume the path has been fixed by wcd_fixpath(). Only forward slashes. */
-   if ((strlen(path) > 2) && (wcd_is_slash(*path)) && (wcd_is_slash(*(path+1))))
-   {
+   if ((strlen(path) > 2) && (wcd_is_slash(*path)) && (wcd_is_slash(*(path+1)))) {
       char *ptr = strchr(path+2,'/');
       if (ptr == NULL)
         return(0);
-      else
-      {
+      else {
         ptr = strchr(ptr+1,'/');
         if (ptr == NULL)
-        {
            return(1);
-        }
         else
            return(0);
       }
@@ -294,9 +288,9 @@ int wcd_islink(const char *dir, int quiet)
    DWORD attrs;
    DWORD dw;
 #ifdef WCD_UNICODE
-   static wchar_t wstr[DD_MAXPATH];
+   static wchar_t wstr[WCD_MAXPATH];
 
-   if (utf8towcs(wstr, dir, DD_MAXPATH) == (size_t)(-1))
+   if (utf8towcs(wstr, dir, WCD_MAXPATH) == (size_t)(-1))
       attrs = INVALID_FILE_ATTRIBUTES;
    else
       attrs = GetFileAttributesW(wstr);
@@ -304,10 +298,8 @@ int wcd_islink(const char *dir, int quiet)
    attrs = GetFileAttributes(dir);
 #endif
 
-   if (attrs == INVALID_FILE_ATTRIBUTES)
-   {
-      if ( !quiet )
-      {
+   if (attrs == INVALID_FILE_ATTRIBUTES) {
+      if ( !quiet ) {
          dw = GetLastError();
          print_error("wcd_islink(): ");
          wcd_PrintError(dw);
@@ -328,18 +320,17 @@ char *wcd_getcwd(char *buf, size_t size)
    DWORD err;
    DWORD dw;
 #ifdef WCD_UNICODE
-   static wchar_t wstr[DD_MAXPATH];
+   static wchar_t wstr[WCD_MAXPATH];
 
-   err = GetCurrentDirectoryW(size, wstr);
+   err = GetCurrentDirectoryW((DWORD)size, wstr);
    if ( err != 0)
-      if (wcstoutf8(buf, wstr, DD_MAXPATH) == (size_t)(-1))
+      if (wcstoutf8(buf, wstr, WCD_MAXPATH) == (size_t)(-1))
          err = 0;
 #else
    err = GetCurrentDirectory(size, buf);
 #endif
 
-   if (err == 0)
-   {
+   if (err == 0) {
      dw = GetLastError();
      print_error("%s", _("Unable to get current working directory: "));
      wcd_PrintError(dw);
@@ -349,28 +340,26 @@ char *wcd_getcwd(char *buf, size_t size)
       return(buf);   /* success */
 }
 
-int wcd_chdir(char *buf, int quiet)
+int wcd_chdir(const char *path, int quiet)
 {
    BOOL err;
    DWORD dw;
 #ifdef WCD_UNICODE
-   static wchar_t wstr[DD_MAXPATH];
+   static wchar_t wstr[WCD_MAXPATH];
 
-   if (utf8towcs(wstr, buf, DD_MAXPATH) == (size_t)(-1))
+   if (utf8towcs(wstr, path, WCD_MAXPATH) == (size_t)(-1))
       err = 0;
    else
       err = SetCurrentDirectoryW(wstr);
 #else
-   err = SetCurrentDirectory(buf);
+   err = SetCurrentDirectory(path);
 #endif
 
-   if (err == 0)
-   {
-      if ( !quiet )
-      {
+   if (err == 0) {
+      if ( !quiet ) {
          dw = GetLastError();
          print_error("");
-         wcd_printf(_("Unable to change to directory %s: "), buf);
+         wcd_printf(_("Unable to change to directory %s: "), path);
          wcd_PrintError(dw);
       }
       return(1);   /* fail */
@@ -379,73 +368,82 @@ int wcd_chdir(char *buf, int quiet)
       return(0);   /* success */
 }
 
-int wcd_mkdir(char *buf, int quiet)
+int wcd_mkdir(const char *path, int quiet)
 {
    BOOL err;
    DWORD dw;
 #ifdef WCD_UNICODE
-   static wchar_t wstr[DD_MAXPATH];
+   static wchar_t wstr[WCD_MAXPATH];
 
-   if (utf8towcs(wstr, buf, DD_MAXPATH) == (size_t)(-1))
+   if (utf8towcs(wstr, path, WCD_MAXPATH) == (size_t)(-1))
       err = FALSE;
    else
       err = CreateDirectoryW(wstr, NULL);
 #else
-   err = CreateDirectory(buf, NULL);
+   err = CreateDirectory(path, NULL);
 #endif
 
    if (err == TRUE)
       return(0);  /* success */
-   else
-   {
-     if ( !quiet )
-     {
+   else {
+     if ( !quiet ) {
        dw = GetLastError();
        print_error("");
-       wcd_printf(_("Unable to create directory %s: "), buf);
+       wcd_printf(_("Unable to create directory %s: "), path);
        wcd_PrintError(dw);
      }
      return(1);  /* fail */
    }
 }
 
-int wcd_rmdir(char *buf, int quiet)
+int wcd_rmdir(const char *path, int quiet)
 {
    BOOL err;
    DWORD dw;
 #ifdef WCD_UNICODE
-   static wchar_t wstr[DD_MAXPATH];
+   static wchar_t wstr[WCD_MAXPATH];
 
-   if (utf8towcs(wstr, buf, DD_MAXPATH) == (size_t)(-1))
+   if (utf8towcs(wstr, path, WCD_MAXPATH) == (size_t)(-1))
       err = FALSE;
    else
       err = RemoveDirectoryW(wstr);
 #else
-   err = RemoveDirectory(buf);
+   err = RemoveDirectory(path);
 #endif
 
    if (err == TRUE)
       return(0);  /* success */
-   else
-   {
-     if ( !quiet )
-     {
+   else {
+     if ( !quiet ) {
        dw = GetLastError();
        print_error("");
-       wcd_printf(_("Unable to remove directory %s: "), buf);
+       wcd_printf(_("Unable to remove directory %s: "), path);
        wcd_PrintError(dw);
      }
      return(1);  /* fail */
    }
 }
 
+int wcd_unlink(const char *path)
+{
+#ifdef WCD_UNICODE
+   wchar_t pathw[WCD_MAXPATH];
+   if (utf8towcs(pathw, path, WCD_MAXPATH) == (size_t)(-1))
+      return -1;
+   return _wunlink(pathw);
+#else
+   return unlink(path);
+#endif
+}
+
+
 /******************************************************************
  *
  * int wcd_isdir(char *dir)
  *
  * test if *dir points to a directory.
  *
- * returns 0 on success, -1 when it fails.
+ * returns 1 when *dir is a directory path, 0 when it isn't.
  *
  * - The following method using POSIX API fails on UNC share paths
  *   like //servername/sharename on MS-Windows systems.
@@ -474,71 +472,57 @@ int wcd_isdir(char *dir, int quiet)
    struct stat buf;
 #endif
 
-   if (wcd_isSharePath(dir))
-   {
-      char tmp[DD_MAXDIR];
+   if (wcd_isSharePath(dir)) {
+      char tmp[WCD_MAXDIR];
       wcd_getcwd(tmp, sizeof(tmp)); /* remember current dir */
 
-      if (wcd_chdir(dir, quiet) == 0) /* just try to change to dir */
-      {
+      if (wcd_chdir(dir, quiet) == 0) { /* just try to change to dir */
         wcd_chdir(tmp, quiet); /* go back */
-        return(0);
+        return 1 ;
       }
       else
-         return(-1);
+         return 0;
    } else {
 #ifdef WCD_UTF16
-      static wchar_t wstr[DD_MAXPATH];
-      if (utf8towcs(wstr, dir, DD_MAXPATH) == (size_t)(-1))
+      static wchar_t wstr[WCD_MAXPATH];
+      if (utf8towcs(wstr, dir, WCD_MAXPATH) == (size_t)(-1))
          err = FALSE;
       else
          err = TRUE;
 
-      if (err == TRUE)
-      {
-         if (_wstat(wstr, &buf) == 0)
-         {
+      if (err == TRUE) {
+         if (_wstat(wstr, &buf) == 0) {
             if (S_ISDIR(buf.st_mode))
-               return(0);
+               return 1;
             else
-               return(-1);
-         }
-         else
-         {
-            if (!quiet)
-            {
+               return(0);
+         } else {
+            if (!quiet) {
               char *errstr = strerror(errno);
               wcd_printf("Wcd: %s: %s\n", dir, errstr);
             }
-            return(-1);
+            return 0;
          }
-      }
-      else
-      {
-        if ( !quiet )
-        {
+      } else {
+        if ( !quiet ) {
           dw = GetLastError();
           wcd_printf("Wcd: %s: ", dir);
           wcd_PrintError(dw);
         }
-        return(-1);  /* fail */
+        return 0;  /* fail */
       }
 #else
-      if (stat(dir, &buf) == 0)
-      {
+      if (stat(dir, &buf) == 0) {
          if (S_ISDIR(buf.st_mode))
-            return(0);
+            return 1;
          else
-            return(-1);
-      }
-      else
-      {
-         if (!quiet)
-         {
+            return(0);
+      } else {
+         if (!quiet) {
            char *errstr = strerror(errno);
            print_error("%s: %s\n", dir, errstr);
          }
-         return(-1);
+         return 0;
       }
 #endif
    }
@@ -551,29 +535,23 @@ int wcd_isdir(char *dir, int quiet)
 
 #if defined(UNIX) || defined(__DJGPP__) || defined(__EMX__)
 
-int wcd_mkdir(char *buf, mode_t m, int quiet)
+int wcd_mkdir(const char *path, mode_t m, int quiet)
 {
-  int err = mkdir(buf, m);
+  int err = mkdir(path, m);
 
   if ( !quiet && err)
-  {
-    char *errstr = strerror(errno);
-    print_error(_("Unable to create directory %s: %s\n"), buf, errstr);
-  }
+    print_error(_("Unable to create directory %s: %s\n"), path, strerror(errno));
   return(err);
 }
 
 #else
 
-int wcd_mkdir(char *buf, int quiet)
+int wcd_mkdir(const char *path, int quiet)
 {
-  int err = mkdir(buf);
+  int err = mkdir(path);
 
   if ( !quiet && err)
-  {
-    char *errstr = strerror(errno);
-    print_error(_("Unable to create directory %s: %s\n"), buf, errstr);
-  }
+    print_error(_("Unable to create directory %s: %s\n"), path, strerror(errno));
   return(err);
 }
 
@@ -588,42 +566,34 @@ int wcd_mkdir(char *buf, int quiet)
 char *replace_volume_path_HOME(char *buf, size_t size)
 {
    static char *home = NULL;      /* value of $HOME env variable */
-   static char home_abs[DD_MAXPATH];  /* absolute volume path of $HOME */
+   static char home_abs[WCD_MAXPATH];  /* absolute volume path of $HOME */
    static char status = 0;
    static size_t  len_home = 0;
    static size_t  len_home_abs = 0;
    size_t i, len_buf;
    size_t j;
-   char tmp[DD_MAXPATH];
-   static char pattern[DD_MAXPATH];
+   char tmp[WCD_MAXPATH];
+   static char pattern[WCD_MAXPATH];
    char *ptr1, *ptr2;
 
 
    if (buf == NULL)
       return(NULL);
 
-   if ( status == 0 )
-   {
+   if ( status == 0 ) {
    /* not intialised. */
    /* initialize only once. */
-      if ((home = getenv("HOME")) != NULL )
-      {
+      if ((home = getenv("HOME")) != NULL ) {
          getcwd(tmp, sizeof(tmp)); /* remember current dir */
-         if (wcd_chdir(home,0) == 0)
-         {
-            if (getcwd(home_abs, sizeof(home_abs)) == NULL)
-            {
+         if (wcd_chdir(home,0) == 0) {
+            if (getcwd(home_abs, sizeof(home_abs)) == NULL) {
                status = 3; /* fail */
-            } else
-            {
-               if (strcmp(home,home_abs) == 0)
-               {
+            } else {
+               if (strcmp(home,home_abs) == 0) {
                   status = 4; /* home and home_abs are equal */
                } else {
-                  if ( ((ptr1 = strrchr(home,DIR_SEPARATOR)) != NULL) && ((ptr2 = strrchr(home_abs,DIR_SEPARATOR)) != NULL))
-                  {
-                     if (strcmp(ptr1,ptr2) == 0)
-                     {
+                  if ( ((ptr1 = strrchr(home,DIR_SEPARATOR)) != NULL) && ((ptr2 = strrchr(home_abs,DIR_SEPARATOR)) != NULL)) {
+                     if (strcmp(ptr1,ptr2) == 0) {
                          /* Both paths have same user name. Strip user name, so that
                           * also paths from other users benefit if users have a common
                           * volume prefix. Depends on how Volume Manager was configured. */
@@ -651,32 +621,26 @@ char *replace_volume_path_HOME(char *buf, size_t size)
       }
       /* printf("status = %d\n", status); */
    }
-   if ( status == 1 )  /* $HOME is shorter or equal length than volume name */
-   {
-      if (dd_match(buf, pattern , 0))
-      {
+   if ( status == 1 ) { /* $HOME is shorter or equal length than volume name */
+      if (dd_match(buf, pattern , 0)) {
          len_buf = strlen(buf);
          for (i=0; i < len_home; i++)
             buf[i] = home[i];
          j = i;
-         for (i=len_home_abs; i < len_buf; i++)
-         {
+         for (i=len_home_abs; i < len_buf; i++) {
             buf[j] = buf[i];
             ++j;
          }
          buf[j] = '\0';
       }
    }
-   if ( status == 2 )  /* $HOME is longer than volume name */
-   {
-      if (dd_match(buf, pattern, 0))
-      {
+   if ( status == 2 ) { /* $HOME is longer than volume name */
+      if (dd_match(buf, pattern, 0)) {
          len_buf = strlen(buf);
          for (i=0; (i < len_home) && (i < size); i++)
             tmp[i] = home[i];
          j = i;
-         for (i=len_home_abs; (i < len_buf) && (i < size); i++)
-         {
+         for (i=len_home_abs; (i < len_buf) && (i < size); i++) {
             tmp[j] = buf[i];
             ++j;
          }
@@ -691,8 +655,7 @@ char *replace_volume_path_HOME(char *buf, size_t size)
 char *wcd_getcwd(char *buf, size_t size)
 {
    char *err = getcwd(buf, size);
-   if ( err == NULL)
-   {
+   if ( err == NULL) {
      char *errstr = strerror(errno);
      print_error(_("Unable to get current working directory: %s\n"), errstr);
    }
@@ -703,61 +666,52 @@ char *wcd_getcwd(char *buf, size_t size)
    return(err);
 }
 
-int wcd_chdir(char *buf, int quiet)
+int wcd_chdir(const char *path, int quiet)
 {
-  int err = chdir(buf);
+  int err = chdir(path);
 
-  if ( !quiet && err)
-  {
-    char *errstr = strerror(errno);
-    print_error(_("Unable to change to directory %s: %s\n"), buf, errstr);
-  }
+  if (!quiet && err)
+    print_error(_("Unable to change to directory %s: %s\n"), path, strerror(errno));
   return(err);
 }
 
-int wcd_rmdir(char *buf, int quiet)
+int wcd_rmdir(const char *path, int quiet)
 {
-  int err = rmdir(buf);
+  int err = rmdir(path);
 
-  if ( !quiet && err)
-  {
-    char *errstr = strerror(errno);
-    print_error(_("Unable to remove directory %s: %s\n"), buf, errstr);
-  }
+  if (!quiet && err)
+    print_error(_("Unable to remove directory %s: %s\n"), path, strerror(errno));
   return(err);
 }
 
+int wcd_unlink(const char *path)
+{
+   return unlink(path);
+}
+
 /******************************************************************
  *
  * int wcd_isdir(char *dir)
  *
  * test if *dir points to a directory.
  *
- * returns 0 on success, -1 when it fails.
+ * returns 1 when *dir is a directory path, 0 when it isn't.
  *
  ******************************************************************/
 int wcd_isdir(char *dir, int quiet)
 {
    struct stat buf;
 
-   if (stat(dir, &buf) == 0)
-   {
+   if (stat(dir, &buf) == 0) {
       if (S_ISDIR(buf.st_mode))
-         return(0);
+         return 1;
       else
-         return(-1);
-   }
-   else
-   {
+         return 0;
+   } else {
       if (!quiet)
-      {
-        char *errstr = strerror(errno);
-        print_error("%s: %s\n", dir, errstr);
-      }
-      return(-1);
+         print_error("%s: %s\n", dir, strerror(errno));
+      return 0;
    }
 }
 
-
 #endif
-
